@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2013 Stephen Colebourne
+ *  Copyright 2001-2014 Stephen Colebourne
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -19,7 +19,6 @@ import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
-import java.io.PrintStream;
 import java.lang.reflect.Modifier;
 import java.security.AllPermission;
 import java.security.CodeSource;
@@ -29,6 +28,7 @@ import java.security.Permissions;
 import java.security.Policy;
 import java.security.ProtectionDomain;
 import java.text.DateFormatSymbols;
+import java.util.Date;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.Locale;
@@ -280,12 +280,14 @@ public class TestDateTimeZone extends TestCase {
             String value = map.get(key);
             TimeZone juZone = TimeZone.getTimeZone(key);
             DateTimeZone zone = DateTimeZone.forTimeZone(juZone);
-            assertEquals(value, zone.getID());
+            assertEquals(DateTimeZone.forID(value), zone);
 //            System.out.println(juZone);
 //            System.out.println(juZone.getDisplayName());
 //            System.out.println(zone);
 //            System.out.println("------");
         }
+        // gee thanks time-zone db maintainer for damaging the database
+        // and breaking the long-standing  association with CAT/EAT
     }
 
     //-----------------------------------------------------------------------
@@ -392,6 +394,41 @@ public class TestDateTimeZone extends TestCase {
         assertEquals("America/New_York", zone.getID());
     }
 
+    public void testFromTimeZoneInvalid() throws Exception {
+        TimeZone jdkZone = new TimeZone() {
+            private static final long serialVersionUID = 1L;
+            @Override
+            public String getID() {
+                return null;
+            }
+            @Override
+            public int getOffset(int era, int year, int month, int day, int dayOfWeek, int milliseconds) {
+                return 0;
+            }
+            @Override
+            public void setRawOffset(int offsetMillis) {
+            }
+            @Override
+            public int getRawOffset() {
+                return 0;
+            }
+            @Override
+            public boolean useDaylightTime() {
+                return false;
+            }
+            @Override
+            public boolean inDaylightTime(Date date) {
+                return false;
+            }
+        };
+        try {
+            DateTimeZone.forTimeZone(jdkZone);
+            fail();
+        } catch (IllegalArgumentException ex) {
+            // expected
+        }
+    }
+
     public void testTimeZoneConversion() {
         TimeZone jdkTimeZone = TimeZone.getTimeZone("GMT-10");
         assertEquals("GMT-10:00", jdkTimeZone.getID());
@@ -459,22 +496,33 @@ public class TestDateTimeZone extends TestCase {
             assertEquals(ZoneInfoProvider.class, DateTimeZone.getProvider().getClass());
         }
         
-        PrintStream syserr = System.err;
+        try {
+            System.setProperty("org.joda.time.DateTimeZone.Folder", "src/test/resources/tzdata");
+            DateTimeZone.setProvider(null);
+            assertEquals(ZoneInfoProvider.class, DateTimeZone.getProvider().getClass());
+            assertEquals(2, DateTimeZone.getAvailableIDs().size());
+            assertEquals(true, DateTimeZone.getAvailableIDs().contains("UTC"));
+            assertEquals(true, DateTimeZone.getAvailableIDs().contains("CET"));
+            
+        } finally {
+            System.getProperties().remove("org.joda.time.DateTimeZone.Folder");
+            DateTimeZone.setProvider(null);
+            assertEquals(ZoneInfoProvider.class, DateTimeZone.getProvider().getClass());
+            assertEquals(true, DateTimeZone.getAvailableIDs().size() > 2);
+        }
+    }
+
+    public void testProvider_badClassName() {
         try {
             System.setProperty("org.joda.time.DateTimeZone.Provider", "xxx");
-            ByteArrayOutputStream baos = new ByteArrayOutputStream();
-            System.setErr(new PrintStream(baos));
-            
             DateTimeZone.setProvider(null);
             
+        } catch (RuntimeException ex) {
+            // expected
             assertEquals(ZoneInfoProvider.class, DateTimeZone.getProvider().getClass());
-            String str = new String(baos.toByteArray());
-            assertTrue(str.indexOf("java.lang.ClassNotFoundException") >= 0);
         } finally {
-            System.setErr(syserr);
             System.getProperties().remove("org.joda.time.DateTimeZone.Provider");
             DateTimeZone.setProvider(null);
-            assertEquals(ZoneInfoProvider.class, DateTimeZone.getProvider().getClass());
         }
     }
     
@@ -572,26 +620,22 @@ public class TestDateTimeZone extends TestCase {
             DateTimeZone.setNameProvider(null);
             assertEquals(DefaultNameProvider.class, DateTimeZone.getNameProvider().getClass());
         }
-        
-        PrintStream syserr = System.err;
+    }
+
+    public void testNameProvider_badClassName() {
         try {
             System.setProperty("org.joda.time.DateTimeZone.NameProvider", "xxx");
-            ByteArrayOutputStream baos = new ByteArrayOutputStream();
-            System.setErr(new PrintStream(baos));
-            
-            DateTimeZone.setNameProvider(null);
+            DateTimeZone.setProvider(null);
             
+        } catch (RuntimeException ex) {
+            // expected
             assertEquals(DefaultNameProvider.class, DateTimeZone.getNameProvider().getClass());
-            String str = new String(baos.toByteArray());
-            assertTrue(str.indexOf("java.lang.ClassNotFoundException") >= 0);
         } finally {
-            System.setErr(syserr);
             System.getProperties().remove("org.joda.time.DateTimeZone.NameProvider");
-            DateTimeZone.setNameProvider(null);
-            assertEquals(DefaultNameProvider.class, DateTimeZone.getNameProvider().getClass());
+            DateTimeZone.setProvider(null);
         }
-    }        
-    
+    }
+
     public void testNameProviderSecurity() {
         if (OLD_JDK) {
             return;
@@ -1007,7 +1051,7 @@ public class TestDateTimeZone extends TestCase {
         DateTimeZone result = (DateTimeZone) ois.readObject();
         ois.close();
         
-        assertSame(zone, result);
+        assertEquals(zone, result);
     }
 
     public void testCommentParse() throws Exception {
