@@ -504,6 +504,48 @@ public class TestMonthDay_Basics extends TestCase {
         assertEquals(expected, result);
     }
 
+    public void testPlusDays_int_wrapMonth() {
+        MonthDay test = new MonthDay(11, 1, ISOChronology.getInstanceUTC());
+        MonthDay result = test.plusDays(31);
+        MonthDay expected = new MonthDay(12, 2, ISOChronology.getInstanceUTC());
+        assertEquals(expected, result);
+    }
+
+    public void testPlusDays_int_wrapMonthTwice() {
+        MonthDay test = new MonthDay(10, 31, ISOChronology.getInstanceUTC());
+        MonthDay result = test.plusDays(32);
+        MonthDay expected = new MonthDay(12, 2, ISOChronology.getInstanceUTC());
+        assertEquals(expected, result);
+    }
+
+    public void testPlusDays_int_wrapMonthIntoNextYear() {
+        MonthDay test = new MonthDay(12, 1, ISOChronology.getInstanceUTC());
+        MonthDay result = test.plusDays(31);
+        MonthDay expected = new MonthDay(1, 1, ISOChronology.getInstanceUTC());
+        assertEquals(expected, result);
+    }
+
+    public void testPlusDays_int_wrapMonthTwiceIntoNextYear() {
+        MonthDay test = new MonthDay(11, 30, ISOChronology.getInstanceUTC());
+        MonthDay result = test.plusDays(32);
+        MonthDay expected = new MonthDay(1, 1, ISOChronology.getInstanceUTC());
+        assertEquals(expected, result);
+    }
+
+    public void testPlusDays_int_wrap50() {
+        MonthDay test = new MonthDay(5, 15, ISOChronology.getInstanceUTC());
+        MonthDay result = test.plusDays(50);
+        MonthDay expected = new MonthDay(7, 4, ISOChronology.getInstanceUTC());
+        assertEquals(expected, result);
+    }
+
+    public void testPlusDays_int_toLeap() {
+        MonthDay test = new MonthDay(2, 28, ISOChronology.getInstanceUTC());
+        MonthDay result = test.plusDays(1);
+        MonthDay expected = new MonthDay(2, 29, ISOChronology.getInstance());
+        assertEquals(expected, result);
+    }
+
     public void testPlusDays_int_fromLeap() {
         MonthDay test = new MonthDay(2, 29, ISOChronology.getInstanceUTC());
         MonthDay result = test.plusDays(1);
@@ -598,6 +640,41 @@ public class TestMonthDay_Basics extends TestCase {
         assertEquals(expected, result);
     }
 
+    public void testMinusDays_int_wrapMonth() {
+        MonthDay test = new MonthDay(12, 1, ISOChronology.getInstanceUTC());
+        MonthDay result = test.minusDays(30);
+        MonthDay expected = new MonthDay(11, 1, ISOChronology.getInstanceUTC());
+        assertEquals(expected, result);
+    }
+
+    public void testMinusDays_int_wrapMonthTwice() {
+        MonthDay test = new MonthDay(12, 1, ISOChronology.getInstanceUTC());
+        MonthDay result = test.minusDays(31);
+        MonthDay expected = new MonthDay(10, 31, ISOChronology.getInstanceUTC());
+        assertEquals(expected, result);
+    }
+
+    public void testMinusDays_int_wrapMonthIntoLastYear() {
+        MonthDay test = new MonthDay(1, 1, ISOChronology.getInstanceUTC());
+        MonthDay result = test.minusDays(31);
+        MonthDay expected = new MonthDay(12, 1, ISOChronology.getInstanceUTC());
+        assertEquals(expected, result);
+    }
+
+    public void testMinusDays_int_wrapMonthTwiceIntoLastYear() {
+        MonthDay test = new MonthDay(1, 1, ISOChronology.getInstanceUTC());
+        MonthDay result = test.minusDays(32);
+        MonthDay expected = new MonthDay(11, 30, ISOChronology.getInstanceUTC());
+        assertEquals(expected, result);
+    }
+
+    public void testMinusDays_int_toLeap() {
+        MonthDay test = new MonthDay(3, 1, ISOChronology.getInstanceUTC());
+        MonthDay result = test.minusDays(1);
+        MonthDay expected = new MonthDay(2, 29, ISOChronology.getInstance());
+        assertEquals(expected, result);
+    }
+
     public void testMinusDays_int_fromLeap() {
         MonthDay test = new MonthDay(2, 29, ISOChronology.getInstanceUTC());
         MonthDay result = test.minusDays(1);
